"""Implements twitter."""

import argparse
import logging

import tweepy

from utils import twitter_helpers

MAX_LEN_TWEET = 280
MAX_MEDIA_FILES = 4
log = logging.getLogger('twitter-wrapper')
logging.basicConfig(level=logging.INFO)


class Twitter:
    """Implements Twitter wrapper."""

    def __init__(
        self,
        twtr_api_key,
        twtr_api_secret_key,
        twtr_access_token,
        twtr_access_token_secret,
    ):
        """Construct Twitter."""
        if twtr_api_key:
            auth = tweepy.OAuthHandler(twtr_api_key, twtr_api_secret_key)
            auth.set_access_token(twtr_access_token, twtr_access_token_secret)
            self.api = tweepy.API(auth)
            api_me = self.api.me()
            log.info(
                'Created Twitter for @%s (%s).',
                api_me.screen_name,
                api_me.id_str,
            )
        else:
            log.error('Missing twitter API Key etc. Cannot create Twitter.')
            self.api = None

    @staticmethod
    def from_args(description=''):
        """Construct Twitter from Args."""
        parser = argparse.ArgumentParser(description=description)
        for twtr_arg_name in [
            'twtr_api_key',
            'twtr_api_secret_key',
            'twtr_access_token',
            'twtr_access_token_secret',
        ]:
            parser.add_argument(
                '--' + twtr_arg_name,
                type=str,
                required=False,
                default=None,
            )
        args = parser.parse_args()
        return Twitter(
            args.twtr_api_key,
            args.twtr_api_secret_key,
            args.twtr_access_token,
            args.twtr_access_token_secret,
        )

    def tweet(
        self,
        tweet_text,
        status_image_files=None,
        update_user_profile=False,
        profile_image_file=None,
        banner_image_file=None,
    ):
        """Tweet."""
        if status_image_files is None:
            status_image_files = []

        log.info('tweet_text: %s', tweet_text)
        log.info('status_image_files: %s', str(status_image_files))
        log.info('update_user_profile: %s', str(update_user_profile))
        log.info('profile_image_file: %s', str(profile_image_file))
        log.info('banner_image_file: %s', str(banner_image_file))

        if len(tweet_text) > MAX_LEN_TWEET:
            log.error('Tweet text is too long. Not tweeting.')
            return

        if len(status_image_files) > MAX_MEDIA_FILES:
            log.warning('Too many (%d) status image files. Truncating.')
            status_image_files = status_image_files[:MAX_MEDIA_FILES]

        if not self.api:
            log.error('Missing API. Cannot tweet')
            return False

        media_ids = twitter_helpers._upload_media(self.api, status_image_files)
        twitter_helpers._update_status(self.api, tweet_text, media_ids)

        if update_user_profile:
            twitter_helpers._update_profile_description(self.api)

        if profile_image_file:
            self.api.update_profile_image(profile_image_file)
            log.info('Update profile image to %s', profile_image_file)

        if banner_image_file:
            self.api.update_profile_banner(banner_image_file)
            log.info('Update profile banner image to %s', banner_image_file)

        return True



"""Implements twitter helpers."""

import logging

from utils import timex

log = logging.getLogger('twitter-wrapper-helpers')
logging.basicConfig(level=logging.INFO)


def _update_status(api, tweet_text, media_ids):
    if len(media_ids) > 0:
        api.update_status(tweet_text, media_ids=media_ids)
    else:
        api.update_status(tweet_text)


def _upload_media(api, image_files):
    media_ids = []
    for image_file in image_files:
        media_id = api.media_upload(image_file).media_id
        media_ids.append(media_id)
        log.info(
            'Uploaded status image %s to twitter as %s',
            image_file,
            media_id,
        )
    return media_ids


def _update_profile_description(api):
    date_with_timezone = timex.format_current_date_with_timezone()
    description = 'Automatically updated at {date_with_timezone}'.format(
        date_with_timezone=date_with_timezone
    )
    api.update_profile(description=description)
    log.info('Updated profile description to: %s', description)



"""Various helpers for MatPlotLib"""

import logging

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.patches import Patch

logging.basicConfig(level=logging.INFO)
log = logging.getLogger('utils.plotx')


class DEFAULTS:
    FONT_FAMILY = 'Futura'
    FONT_SIZE = 12
    COLOR = 'black'
    COLOR_STROKE = 'lightgray'
    COLOR_FILL = (0.9, 0.9, 0.9, 0.5)
    VERTICAL_ALIGNMENT = 'center'
    HORIZONTAL_ALIGNMENT = 'center'
    STROKE_WIDTH = 0.3


mpl.rc('font', family=DEFAULTS.FONT_FAMILY)


def get_circle(
    cxy,
    r,
    fill=DEFAULTS.COLOR_FILL,
    stroke=DEFAULTS.COLOR_STROKE,
    stroke_width=DEFAULTS.STROKE_WIDTH,
):
    do_fill = fill is not None
    return plt.Circle(
        cxy,
        r,
        edgecolor=stroke,
        facecolor=fill,
        linewidth=stroke_width,
        fill=do_fill,
    )


def get_color_patch(color):
    """Draw color patch."""
    return Patch(color=color)


def draw_text(
    xy,
    text,
    verticalalignment=DEFAULTS.VERTICAL_ALIGNMENT,
    horizontalalignment=DEFAULTS.HORIZONTAL_ALIGNMENT,
    fontname=DEFAULTS.FONT_FAMILY,
    fontsize=DEFAULTS.FONT_SIZE,
    fontcolor=DEFAULTS.COLOR,
):
    """Draw text."""
    x, y = xy
    plt.text(
        x,
        y,
        text,
        verticalalignment=verticalalignment,
        horizontalalignment=horizontalalignment,
        fontsize=fontsize,
        fontname=fontname,
        color=fontcolor,
    )


def draw_circle(
    cxy,
    r,
    fill=DEFAULTS.COLOR_FILL,
    stroke=DEFAULTS.COLOR_STROKE,
    stroke_width=DEFAULTS.STROKE_WIDTH,
):
    """Draw circle."""
    ax = plt.gca()
    ax.add_patch(
        get_circle(
            cxy,
            r,
            stroke=stroke,
            fill=fill,
            stroke_width=stroke_width,
        )
    )


def draw_infographic(
    title,
    subtitle,
    footer_text,
    image_file,
    func_plot_inner,
):
    """Draw infographic."""
    plt.axes([0, 0, 1, 1])
    draw_text((0.5, 0.95), title, fontsize=24)
    draw_text((0.5, 0.9), subtitle, fontsize=12)
    draw_text((0.5, 0.05), footer_text, fontsize=8, fontcolor='gray')

    func_plot_inner()

    fig = plt.gcf()
    fig.set_size_inches(16, 9)

    plt.savefig(image_file)
    log.info('Saved infographic to %s', image_file)
